#
#
#   CMFReportTool - Generating PDF-Skins on the Fly from CMF-Sites
#
#   Copyright (C) 2002  net-labs Systemhaus GmbH
#                                               info@net-labs.de
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Portal report tool.
$Id: ReportTool.py,v 1.2 2002/10/30 00:03:55 ueck Exp $
"""
__version__='$Revision: 1.2 $'[11:-2]


import Globals
from Globals import DTMLFile, PersistentMapping
from Acquisition import aq_base,aq_parent
from DateTime import DateTime
from AccessControl import ClassSecurityInfo,getSecurityManager
from OFS.Folder import Folder

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase

from Products.PageTemplates.Expressions import restrictedTraverse
from StructuredText.ST import Basic as STX_Basic
from StructuredText.StructuredText import Document as STX_Document

from StringIO import StringIO
import xml.dom.minidom

import urllib,os.path


from RenderPDF.Parser import TemplateParser,DocumentParser
from STXParser import STXParser



def Items(**kw): return kw


def modifiedOptions():
	# Remove the existing "Properties" option and add our own.
	rval = []
	pos = -1
	for o in Folder.manage_options:
		label = o.get('label', None)
		if label != 'Properties':
			rval.append(o)
	rval[1:1] = [{'label':'Properties',
				  'action':'manage_propertiesForm'}]
	return tuple(rval)


class ZODBResourceHandler:
	''' Wrapper for ZODB Resources and files'''

	def __init__(self,context=None,resourcePath=None):
		self.context = context
		self.resourcePath = resourcePath
		
	def get(self,url):

		typ,url = urllib.splittype(url)
		if url.startswith('//'):
			url = url[2:]
		path = urllib.splitquery(url)[0]

		typ = typ or 'zodb'

		if typ == 'zodb':
			return self.handleZODB(path)
		elif typ == 'file':
			return self.handleFile(path)

		raise ValueError,'urlscheme not supported'
		
	def handleFile(self,path):
		return open(os.sep.join(filter(None,os.path.join([self.resourcePath,path]))),'rb')
		
	def handleZODB(self,path):

		path = path.split('/')
		obj = restrictedTraverse(self.context,path,getSecurityManager())

		# check type and e.g. call object if script ...

		## for OFS.Image-like objects
		if hasattr(obj,'data'):
			obj = obj.data
		
		return StringIO(str(obj))


		
class ReportTool(UniqueObject, Folder, ActionProviderBase):
	'''
	This tool is used to supply reports to a portal.
	'''

	id = 'portal_report'
	meta_type = 'CMF Report Tool'

	_actions = []
	security = ClassSecurityInfo()
		
	_v_templatecache = {}

	resourcePath = None


	manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
					 , 
					 ) + ActionProviderBase.manage_options + modifiedOptions()

	#
	#   ZMI methods
	#
	security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_overview' )
	manage_overview = DTMLFile( 'dtml/explainReportTool', globals() )

	request_varname = 'portal_report'

	security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_propertiesForm')
	manage_propertiesForm = DTMLFile('dtml/ReportToolProps', globals())


	security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_properties')
	def manage_properties(self, request_varname='', REQUEST=None):
		'''
		Changes portal_types properties.
		'''
		self.request_varname = str(request_varname)
		self._v_templatecache = {}	
		if REQUEST is not None:

			return self.manage_propertiesForm(
				self, REQUEST, manage_tabs_message='Properties changed.')
	
	security.declareProtected(CMFCorePermissions.View, 'renderPDF')
	def renderPDF(self,templatename,document_xml,encoding='iso-8859-1',context=None):
		"""
			Render document using template
		"""
		
		context = context or self
		rhandler = ZODBResourceHandler(context,self.resourcePath)
		
		template = self._v_templatecache.get(templatename,None)
		if not template:
			template_xml = self._getOb(templatename)()
			template_dom = xml.dom.minidom.parseString(template_xml)
			template = TemplateParser(template_dom,encoding,resourceHandler=rhandler)()
			self._v_templatecache[templatename] = template
		
		document_dom = xml.dom.minidom.parseString(document_xml)
		document = DocumentParser(document_dom,encoding,resourceHandler=rhandler)

		buf = StringIO()
		document(template,buf)
		buf.seek(0)
		return buf.read()
		

	security.declareProtected(CMFCorePermissions.View, 'convertSTX')
	def convertSTX(self,structuredString,header=1,level=1):
		"""
			convert STX-Text to PML
		"""
		st = STX_Basic(structuredString)
		doc = STX_Document(st)
		return STXParser(doc,header=header,level=level)
		
	

Globals.InitializeClass(ReportTool)
